@@ -14,8 +14,7 @@
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-"""
-|g| optimization algorithm package
-Contains wrappers to algorithm libraries
+"""|g| optimization algorithm package.
 
+Contains wrappers to algorithm libraries.
 """
