@@ -23,19 +23,15 @@
 Abstraction for workflow
 ************************
 """
-from __future__ import absolute_import, division, print_function, unicode_literals
+from __future__ import division, unicode_literals
 
+import logging
 from uuid import uuid4
 
-from future import standard_library
-
 from gemseo.core.discipline import MDODiscipline
 from gemseo.utils.py23_compat import OrderedDict  # automatically dict from py36
 
-standard_library.install_aliases()
-
-
-from gemseo import LOGGER
+LOGGER = logging.getLogger(__name__)
 
 STATUS_FAILED = MDODiscipline.STATUS_FAILED
 STATUS_DONE = MDODiscipline.STATUS_DONE
@@ -44,14 +40,12 @@ STATUS_RUNNING = MDODiscipline.STATUS_RUNNING
 
 
 class ExecutionSequence(object):
-    """
-    A base class for execution sequences.
-    The execution sequence structure is introduced to reflect the main workflow
-    implicitly executed by |g| regarding the given
-    scenario/formulation executed. That structure allows to identify single
-    executions of a same discipline that may be run several times at various
-    stages in the given scenario/formulation.
+    """A base class for execution sequences.
 
+    The execution sequence structure is introduced to reflect the main workflow
+    implicitly executed by |g| regarding the given scenario/formulation executed. That
+    structure allows to identify single executions of a same discipline that may be run
+    several times at various stages in the given scenario/formulation.
     """
 
     START_STR = "["
@@ -68,65 +62,52 @@ class ExecutionSequence(object):
         self._parent = None
 
     def accept(self, visitor):
-        """
-        Accept a visitor object (see Visitor pattern).
-        Have to be implemented by subclasses
+        """Accept a visitor object (see Visitor pattern). Have to be implemented by
+        subclasses.
 
         :param visitor: a visitor object
-
         """
         raise NotImplementedError()
 
     def set_observer(self, obs):
-        """
-        Register the given observer object which is intended to be notified
-        via its update() method each time an underlying
-        discipline changes its status.
-        To be implemented in subclasses.
+        """Register the given observer object which is intended to be notified via its
+        update() method each time an underlying discipline changes its status. To be
+        implemented in subclasses.
 
         :returns: the disciplines list.
-
         """
         raise NotImplementedError()
 
     @property
     def status(self):
-        """
-        Get status value.
+        """Get status value.
 
         :returns: the status value (MDODiscipline.STATUS_XXX values).
-
         """
         return self._status
 
     @status.setter
     def status(self, status):
-        """
-        Set status value.
-
-        :param the status (MDODiscipline.STATUS_XXX values).
+        """Set status value.
 
+        :param status: (MDODiscipline.STATUS_XXX values).
         """
         self._status = status
 
     @property
     def parent(self):
-        """
-        Get the containing execution sequence
+        """Get the containing execution sequence.
 
         :returns: a composite execution sequence.
-
         """
         return self._parent
 
     @parent.setter
     def parent(self, parent):
-        """
-        Set the containing execution sequence as parent.
-        self should be included in parent.sequence_list
+        """Set the containing execution sequence as parent. self should be included in
+        parent.sequence_list.
 
         :returns: the status value.
-
         """
         if self not in parent.sequence_list:
             raise RuntimeError(
@@ -135,27 +116,19 @@ class ExecutionSequence(object):
         self._parent = parent
 
     def enabled(self):
-        """
-        Get activation state.
+        """Get activation state.
 
         :returns: boolean True if enabled.
-
         """
         return self._enabled
 
     def enable(self):
-        """
-        Set the execution sequence as activated (enabled).
-
-        """
+        """Set the execution sequence as activated (enabled)."""
         self.status = STATUS_PENDING
         self._enabled = True
 
     def disable(self):
-        """
-        Set the execution sequence as deactivated (disabled).
-
-        """
+        """Set the execution sequence as deactivated (disabled)."""
         self._enabled = False
 
     def _compute_disc_to_uuids(self):
@@ -171,11 +144,7 @@ class ExecutionSequence(object):
 
 
 class AtomicExecSequence(ExecutionSequence):
-    """
-    An execution sequence to represent the single execution
-    of a given discipline
-
-    """
+    """An execution sequence to represent the single execution of a given discipline."""
 
     def __init__(self, discipline=None):
         super(AtomicExecSequence, self).__init__(discipline)
@@ -191,56 +160,44 @@ class AtomicExecSequence(ExecutionSequence):
         self.disc_to_uuids = {discipline: [self.uuid]}
         self._observer = None
 
-    def __str__(self, *args, **kwargs):
+    def __str__(self):
         return self.discipline.name + "(" + str(self.status) + ")"
 
-    def __repr__(self, *args, **kwargs):
+    def __repr__(self):
         return (
             self.discipline.name + "(" + str(self.status) + ", " + str(self.uuid) + ")"
         )
 
     def accept(self, visitor):
-        """
-        Accept a visitor object (see Visitor pattern)
+        """Accept a visitor object (see Visitor pattern)
 
         :param visitor: a visitor object implementing visit_atomic() method
-
         """
         visitor.visit_atomic(self)
 
     def set_observer(self, obs):
-        """
-        Register given observer obs to be notified (obs.update())
-        when discipline status changes.
+        """Register given observer obs to be notified (obs.update()) when discipline
+        status changes.
 
         :param obs: the observe object implementing update() method
-
         """
         self._observer = obs
 
     def enable(self):
-        """
-        Subscribe to status changes of the discipline
-        (notified via update_status())
-
-        """
+        """Subscribe to status changes of the discipline (notified via
+        update_status())"""
         super(AtomicExecSequence, self).enable()
         self.discipline.add_status_observer(self)
 
     def disable(self):
-        """
-        Unsubscribe from receiving status changes of the discipline
-
-        """
+        """Unsubscribe from receiving status changes of the discipline."""
         super(AtomicExecSequence, self).disable()
         self.discipline.remove_status_observer(self)
 
     def get_state_dict(self):
-        """
-        Get the dictionary of statuses mapping atom uuid to status
+        """Get the dictionary of statuses mapping atom uuid to status.
 
         :returns: the status
-
         """
 
         return {self.uuid: self.status}
@@ -265,13 +222,11 @@ class AtomicExecSequence(ExecutionSequence):
                 self._observer.update(self)
 
     def force_statuses(self, status):
-        """
-        Force the self status and the status of subsequences without
-        notifying the parent (as the force_status is called by a parent),
-        but notify the observer is status changed.
+        """Force the self status and the status of subsequences without notifying the
+        parent (as the force_status is called by a parent), but notify the observer is
+        status changed.
 
         :param: status value (see MDODiscipline.STATUS_XXX values)
-
         """
         old_status = self._status
         self._status = status
@@ -280,10 +235,9 @@ class AtomicExecSequence(ExecutionSequence):
 
 
 class CompositeExecSequence(ExecutionSequence):
-    """
-    A base class for execution sequence made of other execution sequences.
-    Intented to be subclassed.
+    """A base class for execution sequence made of other execution sequences.
 
+    Intented to be subclassed.
     """
 
     START_STR = "'"
@@ -294,7 +248,7 @@ class CompositeExecSequence(ExecutionSequence):
         self.sequence_list = []
         self.disciplines = []
 
-    def __str__(self, *args, **kwargs):
+    def __str__(self):
         str_out = self.START_STR
         for seq in self.sequence_list:
             str_out += str(seq) + ", "
@@ -302,66 +256,50 @@ class CompositeExecSequence(ExecutionSequence):
         return str_out
 
     def accept(self, visitor):
-        """
-        Accept a visitor object (see Visitor pattern)
-        and then make its children accept it too.
+        """Accept a visitor object (see Visitor pattern) and then make its children
+        accept it too.
 
         :param visitor: a visitor object implementing visit_serial() method
-
         """
         self._accept(visitor)
         for seq in self.sequence_list:
             seq.accept(visitor)
 
     def _accept(self, visitor):
-        """
-        Accept a visitor object (see Visitor pattern).
-        To be specifically implemented by subclasses to call relevant
-        visitor method depending the subclass type.
+        """Accept a visitor object (see Visitor pattern). To be specifically implemented
+        by subclasses to call relevant visitor method depending the subclass type.
 
         :param visitor: a visitor object implementing visit_serial() method
-
         """
         raise NotImplementedError()
 
     def set_observer(self, obs):
-        """
-        Set observer obs to subsequences.
-        Override super.set_observer()
+        """Set observer obs to subsequences. Override super.set_observer()
 
         :param obs: observer object implementing update() method
-
         """
         for seq in self.sequence_list:
             seq.set_observer(obs)
 
     def disable(self):
-        """
-        Unsubscribe subsequences from receiving status changes
-        of disciplines
-
-        """
+        """Unsubscribe subsequences from receiving status changes of disciplines."""
         super(CompositeExecSequence, self).disable()
         for seq in self.sequence_list:
             seq.disable()
 
     def force_statuses(self, status):
-        """
-        Force the self status and the status of subsequences.
+        """Force the self status and the status of subsequences.
 
         params: status value (see MDODiscipline.STATUS_XXX values)
-
         """
         self.status = status
         for seq in self.sequence_list:
             seq.force_statuses(status)
 
     def get_state_dict(self):
-        """
-        Get the dictionary of statuses mapping atom uuid to status
+        """Get the dictionary of statuses mapping atom uuid to status.
 
         :returns: the status
-
         """
         state_dict = {}
         for seq in self.sequence_list:
@@ -369,9 +307,8 @@ class CompositeExecSequence(ExecutionSequence):
         return state_dict
 
     def update_child_status(self, child):
-        """
-        Manage status change of child execution sequences.
-        Propagates status change to the parent (containing execution sequence)
+        """Manage status change of child execution sequences. Propagates status change
+        to the parent (containing execution sequence)
 
         :param child: the child execution sequence (contained in sequence_list)
             whose status has changed
@@ -382,8 +319,7 @@ class CompositeExecSequence(ExecutionSequence):
             self._parent.update_child_status(self)
 
     def _update_child_status(self, child):
-        """
-        Handle child execution change. To be implemented in subclasses.
+        """Handle child execution change. To be implemented in subclasses.
 
         :param child: the child execution sequence (contained in sequence_list)
             whose status has changed
@@ -392,8 +328,8 @@ class CompositeExecSequence(ExecutionSequence):
 
 
 class ExtendableExecSequence(CompositeExecSequence):
-    """
-    A base class for composite execution sequence that are extendable.
+    """A base class for composite execution sequence that are extendable.
+
     Intented to be subclassed.
     """
 
@@ -403,12 +339,9 @@ class ExtendableExecSequence(CompositeExecSequence):
             self.extend(sequence)
 
     def extend(self, sequence):
-        """
-        Extend the execution sequence with another ExecutionSequence
-        or a discipline.
+        """Extend the execution sequence with another ExecutionSequence or a discipline.
 
         :param sequence: another execution sequence or
-
         """
         seq_class = sequence.__class__
         self_class = self.__class__
@@ -438,11 +371,9 @@ class ExtendableExecSequence(CompositeExecSequence):
         return self
 
     def _extend_with_disc_list(self, sequence):
-        """
-        Extend by a list of disciplines
+        """Extend by a list of disciplines.
 
         :param sequence: a list of MDODiscipline objects
-
         """
         seq_list = [AtomicExecSequence(disc) for disc in sequence]
         self.sequence_list.extend(seq_list)
@@ -450,43 +381,35 @@ class ExtendableExecSequence(CompositeExecSequence):
         self.uuid_to_disc.update(uuids_dict)
 
     def _extend_with_atomic_sequence(self, sequence):
-        """
-        Extend by a list of AtomicExecutionSequence
+        """Extend by a list of AtomicExecutionSequence.
 
         :param sequence: a list of MDODiscipline objects
-
         """
         self.sequence_list.append(sequence)
         self.uuid_to_disc[sequence.uuid] = sequence
 
     def _extend_with_same_sequence_kind(self, sequence):
-        """
-        Extend by another ExecutionSequence of same type.
+        """Extend by another ExecutionSequence of same type.
 
         :param sequence: an ExecutionSequence of same type as self
-
         """
         self.sequence_list.extend(sequence.sequence_list)
         self.uuid_to_disc.update(sequence.uuid_to_disc)
 
     def _extend_with_diff_sequence_kind(self, sequence):
-        """
-        Extend by another ExecutionSequence of different type.
+        """Extend by another ExecutionSequence of different type.
 
         :param sequence: an ExecutionSequence of type different from self's one
-
         """
         self.sequence_list.append(sequence)
         self.uuid_to_disc.update(sequence.uuid_to_disc)
 
     def _update_child_status(self, child):
-        """
-        Manage status change of child execution sequences.
-        Done status management is handled in subclasses.
+        """Manage status change of child execution sequences. Done status management is
+        handled in subclasses.
 
         :param child: the child execution sequence (contained in sequence_list)
             whose status has changed
-
         """
         if child.status == STATUS_FAILED:
             self.status = STATUS_FAILED
@@ -496,21 +419,17 @@ class ExtendableExecSequence(CompositeExecSequence):
             self.status = child.status
 
     def _update_child_done_status(self, child):
-        """
-        Handle done status of child execution sequences.
-        To be implemented in subclasses
+        """Handle done status of child execution sequences. To be implemented in
+        subclasses.
 
         :param child: the child execution sequence (contained in sequence_list)
             whose status has changed
-
         """
         raise NotImplementedError()
 
 
 class SerialExecSequence(ExtendableExecSequence):
-    """
-    A class to describe a serial execution of disciplines
-    """
+    """A class to describe a serial execution of disciplines."""
 
     START_STR = "["
     END_STR = "]"
@@ -520,18 +439,14 @@ class SerialExecSequence(ExtendableExecSequence):
         self.exec_index = None
 
     def _accept(self, visitor):
-        """
-        Accept a visitor object (see Visitor pattern)
+        """Accept a visitor object (see Visitor pattern)
 
         :param visitor: a visitor object implementing visit_serial() method
-
         """
         visitor.visit_serial(self)
 
     def enable(self):
-        """
-        Activate first child execution sequence.
-        """
+        """Activate first child execution sequence."""
         super(SerialExecSequence, self).enable()
         self.exec_index = 0
         if self.sequence_list:
@@ -540,12 +455,10 @@ class SerialExecSequence(ExtendableExecSequence):
             raise Exception("Serial execution is empty")
 
     def _update_child_done_status(self, child):
-        """
-        Activate next child to given child execution sequence.
-        Disable itself when all children done.
+        """Activate next child to given child execution sequence. Disable itself when
+        all children done.
 
         :param child: the child execution sequence in done state.
-
         """
         if child.status == STATUS_DONE:
             child.disable()
@@ -558,37 +471,28 @@ class SerialExecSequence(ExtendableExecSequence):
 
 
 class ParallelExecSequence(ExtendableExecSequence):
-    """
-    A class to describe a parallel execution of disciplines
-    """
+    """A class to describe a parallel execution of disciplines."""
 
     START_STR = "("
     END_STR = ")"
 
     def _accept(self, visitor):
-        """
-        Accept a visitor object (see Visitor pattern)
+        """Accept a visitor object (see Visitor pattern)
 
         :param visitor: a visitor object implementing visit_parallel() method
-
         """
         visitor.visit_parallel(self)
 
     def enable(self):
-        """
-        Activate all child execution sequences.
-
-        """
+        """Activate all child execution sequences."""
         super(ParallelExecSequence, self).enable()
         for seq in self.sequence_list:
             seq.enable()
 
     def _update_child_done_status(self, child):  # pylint: disable=unused-argument
-        """
-        Disable itself when all children done.
+        """Disable itself when all children done.
 
         :param child: the child execution sequence in done state.
-
         """
         all_done = True
         for seq in self.sequence_list:
@@ -599,17 +503,13 @@ class ParallelExecSequence(ExtendableExecSequence):
 
 
 class LoopExecSequence(CompositeExecSequence):
-    """
-    A class to describe a loop with a controller discipline and
-    an execution_sequence as iterate.
-    """
+    """A class to describe a loop with a controller discipline and an execution_sequence
+    as iterate."""
 
     START_STR = "{"
     END_STR = "}"
 
     def __init__(self, controller, sequence):
-
-        control = None
         if isinstance(controller, AtomicExecSequence):
             control = controller
         elif not isinstance(controller, MDODiscipline):
@@ -640,29 +540,23 @@ class LoopExecSequence(CompositeExecSequence):
         self.iteration_count = 0
 
     def _accept(self, visitor):
-        """
-        Accept a visitor object (see Visitor pattern)
+        """Accept a visitor object (see Visitor pattern)
 
         :param visitor: a visitor object implementing visit_loop() method
-
         """
         visitor.visit_loop(self)
 
     def enable(self):
-        """
-        Active controller execution sequence
-        """
+        """Active controller execution sequence."""
         super(LoopExecSequence, self).enable()
         self.atom_controller.enable()
         self.iteration_count = 0
 
     def _update_child_status(self, child):
-        """
-        Activate iteration successively regarding controller status.
-        Count iterations regarding iteration_sequence status.
+        """Activate iteration successively regarding controller status. Count iterations
+        regarding iteration_sequence status.
 
         :param child: the child execution sequence in done state.
-
         """
         self.status = self.atom_controller.status
         if child == self.atom_controller:
@@ -681,64 +575,55 @@ class LoopExecSequence(CompositeExecSequence):
 
 
 class ExecutionSequenceFactory(object):
-    """
-    A factory class for ExecutionSequence objects.
+    """A factory class for ExecutionSequence objects.
+
     Allow to create AtomicExecutionSequence, SerialExecutionSequence,
-    ParallelExecutionSequence and LoopExecutionSequence.
-    Main |g| workflow is intended to be expressed with those four
-    ExecutionSequence types
+    ParallelExecutionSequence and LoopExecutionSequence. Main |g| workflow is intended
+    to be expressed with those four ExecutionSequence types
     """
 
     @staticmethod
     def atom(discipline):
-        """Returns a structure representing the execution of a discipline.
-        This function is intended to be called by
-        MDOFormulation.get_expected_workflow methods.
+        """Returns a structure representing the execution of a discipline. This function
+        is intended to be called by MDOFormulation.get_expected_workflow methods.
 
         :param discipline: a discipline
         :returns: the structure used within XDSM workflow representation
-
         """
         return AtomicExecSequence(discipline)
 
     @staticmethod
     def serial(sequence=None):
-        """Returns a structure representing the serial execution of
-        the given disciplines.
-        This function is intended to be called by
+        """Returns a structure representing the serial execution of the given
+        disciplines. This function is intended to be called by
         MDOFormulation.get_expected_workflow methods.
 
         :param sequence: any number of discipline
             or the return value of a serial, parallel or loop call
         :returns: a serial execution sequence
-
         """
         return SerialExecSequence(sequence)
 
     @staticmethod
     def parallel(sequence=None):
-        """Returns a structure representing the parallel execution
-        of the given disciplines.
-        This function is intended to be called by
+        """Returns a structure representing the parallel execution of the given
+        disciplines. This function is intended to be called by
         MDOFormulation.get_expected_workflow methods.
 
         :param sequence: any number of discipline or
             the return value of a serial, parallel or loop call
         :returns: a parallel execution sequence
-
         """
         return ParallelExecSequence(sequence)
 
     @staticmethod
     def loop(control, composite_sequence):
-        """Returns a structure representing a loop execution of a
-        This function is intended to be called by
-        MDOFormulation.get_expected_workflow methods.
+        """Returns a structure representing a loop execution of a This function is
+        intended to be called by MDOFormulation.get_expected_workflow methods.
 
         :param control: the discipline object, controller of the loop
         :param composite_sequence: any number of discipline
             or the return value of a serial, parallel or loop call
         :returns: a loop execution sequence
-
         """
         return LoopExecSequence(control, composite_sequence)
