@@ -21,24 +21,26 @@
 #    OTHER AUTHORS   - MACROSCOPIC CHANGES
 #        :author: Pierre-Jean Barjhoux, Benoit Pauwels - MDOScenarioAdapter
 #                                                        Jacobian computation
-"""
-A Scenario which driver is an optimization algorithm
-****************************************************
-"""
+"""A scenario whose driver is an optimization algorithm."""
 from __future__ import division, unicode_literals
 
 import logging
 from copy import copy, deepcopy
 from datetime import timedelta
 from timeit import default_timer as timer
+from typing import Any, Dict, Iterable, List, Optional, Sequence, Tuple
 
 from numpy import atleast_1d, zeros
+from numpy.core.multiarray import ndarray
 from numpy.linalg import norm
 
+from gemseo.algos.design_space import DesignSpace
 from gemseo.algos.lagrange_multipliers import LagrangeMultipliers
 from gemseo.algos.opt.opt_factory import OptimizersFactory
+from gemseo.algos.opt_result import OptimizationResult
 from gemseo.algos.post_optimal_analysis import PostOptimalAnalysis
 from gemseo.core.discipline import MDODiscipline
+from gemseo.core.execution_sequence import LoopExecSequence
 from gemseo.core.json_grammar import JSONGrammar
 from gemseo.core.parallel_execution import DiscParallelLinearization
 from gemseo.core.scenario import Scenario
@@ -51,73 +53,52 @@ LOGGER = logging.getLogger(__name__)
 
 
 class MDOScenario(Scenario):
-    """Multidisciplinary Design Optimization Scenario, main user interface Creates an
-    optimization problem and solves it with an optimizer.
+    """A multidisciplinary scenario to be executed by an optimizer.
 
-    The main differences between Scenario and MDOScenario are the allowed
-    inputs in the MDOScenario.json, which differs from DOEScenario.json,
-    at least on the driver names
+    A :class:`MDOScenario` is a particular :class:`.Scenario`
+    whose driver is an optimization algorithm.
+    This algorithm must be implemented in an :class:`.OptimizationLibrary`.
 
-    MDO Problem description: links the disciplines and the formulation
-    to create an optimization problem.
-    Use the class by instantiation.
-
-    Create your disciplines beforehand.
-
-    Specify the formulation by giving the class name such as the string
-    "MDF"
-
-    The reference_input_data is the typical input data dict that is provided
-    to the run method of the disciplines
-
-    Specify the objective function name, which must be an output
-    of a discipline of the scenario, with the "objective_name" attribute
-
-    If you want to add additional design constraints,
-    use the add_user_defined_constraint method
-
-    To view the results, use the "post_process" method after execution.
-    You can view:
-
-    - the design variables history, the objective value, the constraints,
-      by using:
-      scenario.post_process("OptHistoryView", show=False, save=True)
-    - Quadratic approximations of the functions close to the
-      optimum, when using gradient based algorithms, by using:
-      scenario.post_process("QuadApprox", method="SR1", show=False,
-      save=True, function="my_objective_name",
-      file_path="appl_dir")
-    - Self Organizing Maps of the design space, by using:
-      scenario.post_process("SOM", save=True, file_path="appl_dir")
-
-    To list post-processing on your setup,
-    use the method scenario.posts
-    For more detains on their options, go to the "gemseo.post" package
+    Attributes:
+        clear_history_before_run (bool): If True, clear history before run.
     """
 
     # Constants for input variables in json schema
     MAX_ITER = "max_iter"
     X_OPT = "x_opt"
 
+    _ATTR_TO_SERIALIZE = Scenario._ATTR_TO_SERIALIZE + (
+        "formulation",
+        "disciplines",
+        "clear_history_before_run",
+        "_algo_factory",
+    )
+
     def __init__(
         self,
-        disciplines,
-        formulation,
-        objective_name,
-        design_space,
-        name=None,
-        **formulation_options
-    ):
-        """Constructor, initializes the MDO scenario Objects instantiation and checks
-        are made before run intentionally.
-
-        :param disciplines: the disciplines of the scenario
-        :param formulation: the formulation name,
-            the class name of the formulation in gemseo.formulations
-        :param objective_name: the objective function name
-        :param design_space: the design space
-        :param name: scenario name
-        :param formulation_options: options for creation of the formulation
+        disciplines,  # type: Sequence[MDODiscipline]
+        formulation,  # type: str
+        objective_name,  # type: str
+        design_space,  # type: DesignSpace
+        name=None,  # type: Optional[str]
+        grammar_type=MDODiscipline.JSON_GRAMMAR_TYPE,  # type: str
+        **formulation_options  # type: Any
+    ):  # type: (...) -> None
+        """
+        Args:
+            disciplines: The disciplines
+                used to compute the objective, constraints and observables
+                from the design variables.
+            formulation: The name of the MDO formulation,
+                also the name of a class inheriting from :class:`.MDOFormulation`.
+            objective_name: The name of the objective.
+            design_space: The design space.
+            name: The name to be given to this scenario.
+                If None, use the name of the class.
+            grammar_type: The type of grammar to use for IO declaration
+                , e.g. JSON_GRAMMAR_TYPE or SIMPLE_GRAMMAR_TYPE.
+            **formulation_options: The options
+                to be passed to the :class:`.MDOFormulation`.
         """
         # This loads the right json grammars from class name
         super(MDOScenario, self).__init__(
@@ -125,13 +106,13 @@ class MDOScenario(Scenario):
             formulation,
             objective_name,
             design_space,
-            name,
+            name=name,
+            grammar_type=grammar_type,
             **formulation_options
         )
         self.clear_history_before_run = False
 
-    def _run_algorithm(self):
-        """Runs the optimization algo."""
+    def _run_algorithm(self):  # type: (...) -> OptimizationResult
         problem = self.formulation.opt_problem
         # Clears the database when multiple runs are performed (bi level)
         if self.clear_history_before_run:
@@ -143,7 +124,7 @@ class MDOScenario(Scenario):
             options = {}
         if self.MAX_ITER in options:
             LOGGER.warning(
-                "Double definition of algorithm option " "max_iter, keeping value: %s",
+                "Double definition of algorithm option max_iter, keeping value: %s",
                 max_iter,
             )
             options.pop(self.MAX_ITER)
@@ -155,7 +136,6 @@ class MDOScenario(Scenario):
         return self.optimization_result
 
     def _run(self):
-        """Execute the scenario and run the optimization problems."""
         t_0 = timer()
         LOGGER.info(" ")
         LOGGER.info("*** Start MDO Scenario execution ***")
@@ -170,60 +150,80 @@ class MDOScenario(Scenario):
         )
 
     def _init_algo_factory(self):
-        """Initalizes the algorithms factory."""
         self._algo_factory = OptimizersFactory()
 
+    def _update_grammar_input(self):  # type: (...) -> None
+        self.input_grammar.update_elements(
+            algo=str, max_iter=int, algo_options=dict, python_typing=True
+        )
+        self.input_grammar.update_required_elements(
+            algo=True, max_iter=True, algo_options=False
+        )
+
 
 class MDOScenarioAdapter(MDODiscipline):
-    """An adapter class for MDO Scenario:
+    """An adapter class for MDO Scenario.
 
-    input variables are specified they update default_data in the top level discipline
-    they output data from the top level discipline outputs.
+    The specified input variables update the default input data
+    of the top level discipline
+    while the output ones filter the output data from the top level discipline outputs.
+
+    Attributes:
+        scenario (Scenario): The scenario to be adapted.
+        post_optimal_analysis (PostOptimalAnalysis): The post-optimal analysis.
     """
 
     LOWER_BND_SUFFIX = "_lower_bnd"
     UPPER_BND_SUFFIX = "_upper_bnd"
     MULTIPLIER_SUFFIX = "_multiplier"
 
+    _ATTR_TO_SERIALIZE = MDODiscipline._ATTR_TO_SERIALIZE + (
+        "scenario",
+        "_inputs_list",
+        "_reset_x0_before_opt",
+        "_x_dict_0",
+        "_set_x0_before_opt",
+        "_set_bounds_before_opt",
+        "_outputs_list",
+        "_output_multipliers",
+    )
+
     def __init__(
         self,
-        scenario,
-        inputs_list,
-        outputs_list,
-        reset_x0_before_opt=False,
-        set_x0_before_opt=False,
-        set_bounds_before_opt=False,
-        cache_type=MDODiscipline.SIMPLE_CACHE,
-        output_multipliers=False,
-    ):
-        """Initialize the scenario adapter.
+        scenario,  # type: Scenario
+        inputs_list,  # type: Sequence[str]
+        outputs_list,  # type: Sequence[str]
+        reset_x0_before_opt=False,  # type: bool
+        set_x0_before_opt=False,  # type: bool
+        set_bounds_before_opt=False,  # type: bool
+        cache_type=MDODiscipline.SIMPLE_CACHE,  # type: str
+        output_multipliers=False,  # type: bool
+        grammar_type=MDODiscipline.JSON_GRAMMAR_TYPE,  # type: str
+        name=None,  # type: Optional[str]
+    ):  # type: (...) -> None
+        """
+        Args:
+            scenario: The scenario to adapt.
+            inputs_list: The inputs to overload at sub-scenario execution.
+            outputs_list: The outputs to get from the sub-scenario execution.
+            reset_x0_before_opt: If True, reset the initial guess
+                before running the sub optimization.
+            set_x0_before_opt: If True, set the initial guess of the sub-scenario.
+                This is useful for multi-start optimization.
+            set_bounds_before_opt: If True, set the bounds of the design space.
+                This is useful for trust regions.
+            cache_type: The type of cache policy.
+            output_multipliers: If True,
+                the Lagrange multipliers of the scenario optimal solution are computed
+                and added to the outputs.
+            name: The name of the scenario adapter.
+                If None, use ``"{}_adapter"``.
 
-        :param scenario: the scenario to adapt
-        :type scenario: MDOScenario
-        :param inputs_list: list of inputs to overload at
-            sub scenario execution
-        :type inputs_list: list(str)
-        :param outputs_list: list of outputs to get from
-            scenario execution
-        :type outputs_list: list(str)
-        :param reset_x0_before_opt: before running the sub optimization, reset
-            the initial guess
-        :type reset_x0_before_opt: bool
-        :param set_x0_before_opt: if True, sets the initial point of the sub
-            scenario, useful for multi-start
-        :type set_x0_before_opt: bool
-        :param set_bounds_before_opt: if True, sets the bounds of the design
-            space, useful for trust regions
-        :type set_bounds_before_opt: bool
-        :param cache_type: type of cache policy, SIMPLE_CACHE
-            or HDF5_CACHE
-        :type cache_type: str
-        :param output_multipliers: if True then the Lagrange multipliers of the
-            scenario optimal solution are computed and added to the outputs
-        :type output_multipliers: bool
+        Raises:
+            ValueError: If both `reset_x0_before_opt` and `set_x0_before_opt` are True.
         """
         if reset_x0_before_opt and set_x0_before_opt:
-            raise ValueError("Inconsistent options for ScenarioAdapter !")
+            raise ValueError("Inconsistent options for MDOScenarioAdapter.")
         self.scenario = scenario
         self._set_x0_before_opt = set_x0_before_opt
         self._set_bounds_before_opt = set_bounds_before_opt
@@ -231,8 +231,10 @@ class MDOScenarioAdapter(MDODiscipline):
         self._outputs_list = outputs_list
         self._reset_x0_before_opt = reset_x0_before_opt
         self._output_multipliers = output_multipliers
-        name = "{}_adapter".format(scenario.name)
-        super(MDOScenarioAdapter, self).__init__(name, cache_type=cache_type)
+        name = name or "{}_adapter".format(scenario.name)
+        super(MDOScenarioAdapter, self).__init__(
+            name, cache_type=cache_type, grammar_type=grammar_type
+        )
 
         self._update_grammars()
         self._dv_in_names = None
@@ -266,8 +268,13 @@ class MDOScenarioAdapter(MDODiscipline):
 
         self.post_optimal_analysis = None
 
-    def _update_grammars(self):
-        """Update the input and output grammars."""
+    def _update_grammars(self):  # type: (...) -> None
+        """Update the input and output grammars.
+
+        Raises:
+            ValueError: Either if a specified input is missing from the input grammar
+                or if a specified output is missing from the output grammar.
+        """
         formulation = self.scenario.formulation
         opt_problem = formulation.opt_problem
         top_leveld = formulation.get_top_level_disc()
@@ -313,21 +320,24 @@ class MDOScenarioAdapter(MDODiscipline):
 
         if missing_out:
             raise ValueError(
-                "Can't compute outputs from scenarios: " + str(missing_out)
+                "Can't compute outputs from scenarios: {}.".format(
+                    ", ".join(sorted(missing_out))
+                )
             )
         missing_inpt = set(self._inputs_list) - set(self.input_grammar.get_data_names())
         if missing_inpt:
             raise ValueError(
-                "Can't compute inputs from scenarios: " + str(missing_inpt)
+                "Can't compute inputs from scenarios: {}.".format(
+                    ", ".join(sorted(missing_inpt))
+                )
             )
 
         # Add the Lagrange multipliers to the output grammar
         if self._output_multipliers:
             self._add_output_multipliers()
 
-    def _add_output_multipliers(self):
-        """Add the Lagrange multipliers of the scenario optimal solution as outputs of
-        the adapter."""
+    def _add_output_multipliers(self):  # type: (...) -> None
+        """Add the Lagrange multipliers of the scenario optimal solution as outputs."""
         # Fill a dictionary with data of typical shapes
         base_dict = dict()
         problem = self.scenario.formulation.opt_problem
@@ -359,16 +369,19 @@ class MDOScenarioAdapter(MDODiscipline):
         self.output_grammar.update_from(multipliers_grammar)
 
     @staticmethod
-    def get_bnd_mult_name(variable_name, is_upper):
+    def get_bnd_mult_name(
+        variable_name,  # type: str
+        is_upper,  # type:bool
+    ):  # type: (...) -> str
         """Return the name of the lower bound-constraint multiplier of a variable.
 
-        :param variable_name: name of the variable
-        :type variable_name: str
-        :param is_upper: if True then return the upper bound-constraint multiplier
-            name, otherwise return the lower bound-constraint multiplier
-        :type is_upper: bool
-        :return: name of the lower bound-constraint multiplier
-        :rtype: str
+        Args:
+            variable_name: The name of the variable.
+            is_upper: If True, return name of the upper bound-constraint multiplier.
+                Otherwise, return the name of the lower bound-constraint multiplier.
+
+        Returns:
+            The name of a bound-constraint multiplier.
         """
         mult_name = variable_name
         mult_name += "_upp-bnd" if is_upper else "_low-bnd"
@@ -376,24 +389,26 @@ class MDOScenarioAdapter(MDODiscipline):
         return mult_name
 
     @staticmethod
-    def get_cstr_mult_name(constraint_name):
+    def get_cstr_mult_name(
+        constraint_name,  # type: str
+    ):  # type: (...) ->str
         """Return the name of the multiplier of a constraint.
 
-        :param constraint_name: name of the constraint
-        :type constraint_name: str
-        :return: name of the multiplier
-        :rtype: str
+        Args:
+            constraint_name: The name of the constraint.
+
+        Returns:
+            The name of the multiplier.
         """
         return constraint_name + MDOScenarioAdapter.MULTIPLIER_SUFFIX
 
-    def _run(self):
-        """Runs the scenario."""
+    def _run(self):  # type: (...) -> None
         self._pre_run()
         self.scenario.execute()
         self._post_run()
 
-    def _pre_run(self):
-        """Pre-processes the scenario."""
+    def _pre_run(self):  # type: (...) -> None
+        """Pre-run the scenario."""
         formulation = self.scenario.formulation
         opt_problem = formulation.opt_problem
         design_space = opt_problem.design_space
@@ -409,6 +424,9 @@ class MDOScenarioAdapter(MDODiscipline):
         # Default inputs have changed, therefore caches shall be cleared
         self.scenario.cache.clear()
         self.scenario.reset_statuses_for_run()
+
+        # Reset the iter counter for the opt problem.
+        opt_problem.current_iter = 0
         for func in opt_problem.get_all_functions():
             # Avoids max_iter reached
             func.n_calls = 0
@@ -433,7 +451,7 @@ class MDOScenarioAdapter(MDODiscipline):
                 upper_bound = self.local_data[name + upper_suffix]
                 design_space.set_upper_bound(name, upper_bound)
 
-    def _post_run(self):
+    def _post_run(self):  # type: (...) -> None
         """Post-process the scenario."""
         formulation = self.scenario.formulation
         opt_problem = formulation.opt_problem
@@ -462,9 +480,12 @@ class MDOScenarioAdapter(MDODiscipline):
         if self._output_multipliers:
             self._compute_lagrange_multipliers()
 
-    def _retrieve_top_level_outputs(self):
-        """Overwrites the adapter outputs with the top-level discipline outputs and
-        optimal design parameters."""
+    def _retrieve_top_level_outputs(self):  # type: (...) -> None
+        """Retrieve the top-level outputs.
+
+        This methods overwrites the adapter outputs with the top-level discipline
+        outputs and the optimal design parameters.
+        """
         formulation = self.scenario.formulation
         opt_problem = formulation.opt_problem
         top_leveld = formulation.get_top_level_disc()
@@ -477,9 +498,11 @@ class MDOScenarioAdapter(MDODiscipline):
             if out_ds is not None:
                 self.local_data[outdata] = out_ds
 
-    def _compute_lagrange_multipliers(self):
-        """Compute the Lagrange multipliers for the optimal solution of the scenario and
-        store them in the local data."""
+    def _compute_lagrange_multipliers(self):  # type: (...) -> None
+        """Compute the Lagrange multipliers for the optimal solution of the scenario.
+
+        This methods stores the multipliers in the local data.
+        """
         # Compute the Lagrange multipliers
         problem = self.scenario.formulation.opt_problem
         x_opt = problem.solution.x_opt
@@ -513,69 +536,77 @@ class MDOScenarioAdapter(MDODiscipline):
             }
         )
 
-    def get_expected_workflow(self):
+    def get_expected_workflow(self):  # type: (...) -> LoopExecSequence
         return self.scenario.get_expected_workflow()
 
-    def get_expected_dataflow(self):
+    def get_expected_dataflow(
+        self,
+    ):  # type: (...) -> List[Tuple[MDODiscipline,MDODiscipline,List[str]]]
         return self.scenario.get_expected_dataflow()
 
-    def _compute_jacobian(self, inputs=None, outputs=None):
-        """
-        Computes the Jacobian of the adapted scenario outputs with respect to
-        its inputs.
-        The Jacobian is stored as a dict of ndarray dict:
-        jac = {name: { input_name: (output_dim, input_dim) ndarray } }
+    def _compute_jacobian(
+        self,
+        inputs=None,  # type: Optional[Sequence[str]]
+        outputs=None,  # type: Optional[Sequence[str]]
+    ):  # type: (...) -> None
+        """Compute the Jacobian of the adapted scenario outputs.
 
-        The bound-constraints on the scenario optimization variables are
-        assumed independent of the other scenario inputs.
+        The Jacobian is stored as a dictionary of numpy arrays:
+        jac = {name: { input_name: ndarray(output_dim, input_dim) } }
 
-        :param inputs: linearization should be performed with respect
-            to inputs list. If None, linearization should
-            be performed wrt all inputs (Default value = None)
-        :param outputs: linearization should be performed on outputs list.
-            If None, linearization should be performed
-            on all outputs (Default value = None)
+        The bound-constraints on the scenario optimization variables
+        are assumed independent of the other scenario inputs.
+
+        Args:
+            inputs: The linearization should be performed with respect to these inputs.
+                If None, the linearization should be performed w.r.t. all inputs.
+            outputs: The linearization should be performed on these outputs.
+                If None, the linearization should be performed on all outputs.
+
+        Raises:
+            ValueError: Either
+                if the dimension of the objective function is greater than 1,
+                if a specified input is not an input of the adapter,
+                if a specified output is not an output of the adapter,
+                or if there is non differentiable outputs.
         """
         opt_problem = self.scenario.formulation.opt_problem
-        ineq_tol = opt_problem.ineq_tolerance
-        outvars = opt_problem.objective.outvars
-        if len(outvars) != 1:
+        objective_names = self.scenario.formulation.opt_problem.objective.outvars
+        if len(objective_names) != 1:
             raise ValueError("The objective must be single-valued.")
 
         # Check the required inputs
-        if inputs is None and not self._set_bounds_before_opt:
-            inputs = self._inputs_list
-        elif inputs is None and self._set_bounds_before_opt:
-            # Bounds are inputs of the adapter
-            inputs = [
-                name for name in self._bounds_names if name not in self._inputs_list
-            ]
-            inputs = self._inputs_list + inputs
-        elif set(inputs) - set(self._inputs_list) - set(self._bounds_names):
+        if inputs is None:
+            inputs = set(self._inputs_list + self._bounds_names)
+        else:
             not_inputs = set(inputs) - set(self._inputs_list) - set(self._bounds_names)
-            raise ValueError(
-                "The following are not inputs of the adapter: "
-                + ", ".join(not_inputs)
-                + "."
-            )
+            if not_inputs:
+                raise ValueError(
+                    "The following are not inputs of the adapter: {}.".format(
+                        ", ".join(sorted(not_inputs))
+                    )
+                )
         # N.B the adapter is assumed constant w.r.t. bounds
         bound_inputs = set(inputs) & set(self._bounds_names)
 
         # Check the required outputs
         if outputs is None:
-            outputs = outvars
-        elif set(outputs) - set(self._outputs_list):
+            outputs = objective_names
+        else:
+            not_outputs = sorted(set(outputs) - set(self._outputs_list))
+            if not_outputs:
+                raise ValueError(
+                    "The following are not outputs of the adapter: {}.".format(
+                        ", ".join(not_outputs)
+                    )
+                )
+
+        non_differentiable_outputs = sorted(set(outputs) - set(objective_names))
+        if non_differentiable_outputs:
             raise ValueError(
-                "The following are not outputs of the adapter: "
-                + str(set(outputs) - set(self._outputs_list))
-                + "."
-            )
-        nondifferentiable_outputs = set(outputs) - set(outvars)
-        if nondifferentiable_outputs:
-            raise ValueError(
-                "Post-optimal Jacobians of "
-                + ", ".join(nondifferentiable_outputs)
-                + " cannot be computed."
+                "Post-optimal Jacobians of {} cannot be computed.".format(
+                    ", ".join(non_differentiable_outputs)
+                )
             )
 
         # Initialize the Jacobian
@@ -587,29 +618,36 @@ class MDOScenarioAdapter(MDODiscipline):
         jacobians = self._compute_auxiliary_jacobians(diff_inputs, use_threading=True)
 
         # Perform the post-optimal analysis
-        self.post_optimal_analysis = PostOptimalAnalysis(opt_problem, ineq_tol)
+        ineq_tolerance = opt_problem.ineq_tolerance
+        self.post_optimal_analysis = PostOptimalAnalysis(opt_problem, ineq_tolerance)
         post_opt_jac = self.post_optimal_analysis.execute(
             outputs, diff_inputs, jacobians
         )
         self.jac.update(post_opt_jac)
 
         # Fill the Jacobian blocks w.r.t. bounds with zeros
-        for out_jac in self.jac.values():
-            for in_name in bound_inputs:
-                in_dim = self.default_inputs[in_name].size
-                out_jac[in_name] = zeros((1, in_dim))
+        for output_derivatives in self.jac.values():
+            for bound_input_name in bound_inputs:
+                bound_input_size = self.default_inputs[bound_input_name].size
+                output_derivatives[bound_input_name] = zeros((1, bound_input_size))
 
-    def _compute_auxiliary_jacobians(self, inputs, func_names=None, use_threading=True):
-        """Computes the Jacobians of the optimization functions.
+    def _compute_auxiliary_jacobians(
+        self,
+        inputs,  # type: Iterable[str]
+        func_names=None,  # type: Optional[Iterable[str]]
+        use_threading=True,  # type: bool
+    ):  # type: (...) -> Dict[str,Dict[str,ndarray]]
+        """Compute the Jacobians of the optimization functions.
 
-        :param inputs: names list of the inputs w.r.t. which differentiate
-        :type inputs: list(str)
-        :param func_names: names list of the functions to differentiate
-            If None then all the optimizations functions are differentiated
-        :type func_names: list(str)
-        :param use_threading : if True, use Threads instead of processes
-            to parallelize the execution
-        :type use_threading: bool
+        Args:
+            inputs: The names of the inputs w.r.t. which differentiate.
+            func_names: The names of the functions to differentiate
+                If None, then all the optimizations functions are differentiated.
+            use_threading: If True, use threads instead of processes
+                to parallelize the execution.
+
+        Returns:
+            The Jacobians of the optimization functions.
         """
         # Gather the names of the functions to differentiate
         opt_problem = self.scenario.formulation.opt_problem
@@ -619,48 +657,49 @@ class MDOScenarioAdapter(MDODiscipline):
             )
 
         # Identify the disciplines that compute the functions
-        disc_dict = dict()
-        for name in func_names:
-            for disc in self.scenario.formulation.get_top_level_disc():
-                if disc.is_all_outputs_existing([name]):
-                    disc_dict[name] = disc
+        disciplines = dict()
+        for func_name in func_names:
+            for discipline in self.scenario.formulation.get_top_level_disc():
+                if discipline.is_all_outputs_existing([func_name]):
+                    disciplines[func_name] = discipline
                     break
 
         # Linearize the required disciplines
-        for disc in set(disc_dict.values()):
-            inputs_set = set(disc.get_input_data_names()) & set(inputs)
-            outputs_set = set(disc.get_output_data_names()) & set(func_names)
-            if inputs_set and outputs_set:
-                disc.add_differentiated_inputs(list(inputs_set))
-                disc.add_differentiated_outputs(list(outputs_set))
-        disc_list = list(set(disc_dict.values()))
-        paralell_lin = DiscParallelLinearization(disc_list, use_threading=use_threading)
+        unique_disciplines = list(set(disciplines.values()))
+        for discipline in unique_disciplines:
+            diff_inputs = set(discipline.get_input_data_names()) & set(inputs)
+            diff_outputs = set(discipline.get_output_data_names()) & set(func_names)
+            if diff_inputs and diff_outputs:
+                discipline.add_differentiated_inputs(list(diff_inputs))
+                discipline.add_differentiated_outputs(list(diff_outputs))
+
+        parallel_linearization = DiscParallelLinearization(
+            unique_disciplines, use_threading=use_threading
+        )
         # Update the local data with the optimal design parameters
         # [The adapted scenario is assumed to have been run beforehand.]
-        x_opt_dict = opt_problem.design_space.get_current_x_dict()
         post_opt_data = copy(self.local_data)
-        post_opt_data.update(x_opt_dict)
-        paralell_lin.execute([post_opt_data] * len(disc_list))
+        post_opt_data.update(opt_problem.design_space.get_current_x_dict())
+        parallel_linearization.execute([post_opt_data] * len(unique_disciplines))
 
         # Store the Jacobians
         jacobians = dict()
-        for name in func_names:
-            jacobians[name] = dict()
+        for func_name in func_names:
+            jacobians[func_name] = dict()
+            func_jacobian = disciplines[func_name].jac[func_name]
             for input_name in inputs:
-                jac_block = disc_dict[name].jac[name].get(input_name)
-                if jac_block is None:
-                    output_value = self.get_outputs_by_name(name)
-                    input_value = self.get_inputs_by_name(input_name)
-                    jac_block = zeros((len(output_value, len(input_value))))
-                jacobians[name][input_name] = jac_block
+                jacobians[func_name][input_name] = func_jacobian[input_name]
 
         return jacobians
 
-    def add_outputs(self, outputs_names):
+    def add_outputs(
+        self,
+        outputs_names,  # type: Iterable[str]
+    ):  # type: (...) -> None
         """Add outputs to the scenario adapter.
 
-        :param outputs_names: names of the outputs to be added
-        :type outputs_names: list(str)
+        Args:
+            outputs_names: The names of the outputs to be added.
         """
         names_to_add = [
             name for name in outputs_names if name not in self._outputs_list
@@ -670,12 +709,9 @@ class MDOScenarioAdapter(MDODiscipline):
 
 
 class MDOObjScenarioAdapter(MDOScenarioAdapter):
-    """A scenario adapter that overwrites the local data with the optimal objective
-    function value."""
+    """A scenario adapter overwriting the local data with the optimal objective."""
 
-    def _retrieve_top_level_outputs(self):
-        """Overwrites the adapter outputs with the top-level discipline outputs and
-        optimal design parameters."""
+    def _retrieve_top_level_outputs(self):  # type: (...) -> None
         formulation = self.scenario.formulation
         opt_problem = formulation.opt_problem
         top_leveld = formulation.get_top_level_disc()
@@ -685,10 +721,9 @@ class MDOObjScenarioAdapter(MDOScenarioAdapter):
         f_opt = opt_problem.get_optimum()[0]
         if not opt_problem.minimize_objective:
             f_opt = -f_opt
-        outvars = opt_problem.objective.outvars
-        if not len(outvars) == 1:
+        if not opt_problem.is_mono_objective:
             raise ValueError("The objective function must be single-valued.")
-        optim_data[outvars[0]] = atleast_1d(f_opt)
+        optim_data[opt_problem.objective.outvars[0]] = atleast_1d(f_opt)
 
         # Overwrite the adapter local data
         for outdata in self._outputs_list:
@@ -699,23 +734,11 @@ class MDOObjScenarioAdapter(MDOScenarioAdapter):
             if out_ds is not None:
                 self.local_data[outdata] = out_ds
 
-    def _compute_jacobian(self, inputs=None, outputs=None):
-        """
-        Computes the Jacobian of the adapted scenario outputs with respect to
-        its inputs.
-        The Jacobian is stored as a dict of ndarray dict:
-        jac = {name: { input_name: (output_dim, input_dim) ndarray } }
-
-        The bound-constraints on the scenario optimization variables are
-        assumed independent of the other scenario inputs.
-
-        :param inputs: linearization should be performed with respect
-            to inputs list. If None, linearization should
-            be performed wrt all inputs (Default value = None)
-        :param outputs: linearization should be performed on outputs list.
-            If None, linearization should be performed
-            on all outputs (Default value = None)
-        """
+    def _compute_jacobian(
+        self,
+        inputs=None,  # type: Optional[Sequence[str]]
+        outputs=None,  # type: Optional[Sequence[str]]
+    ):  # type: (...) -> None
         MDOScenarioAdapter._compute_jacobian(self, inputs, outputs)
         # The gradient of the objective function cannot be computed by the
         # disciplines, but the gradients of the constraints can.
