@@ -99,10 +99,8 @@ from __future__ import division, unicode_literals
 
 import logging
 import pickle
-import re
-from os import makedirs
-from os.path import exists, join
-from typing import Any, Dict, List, Optional, Union
+from copy import deepcopy
+from typing import Any, Dict, Mapping, Optional, Sequence, Union
 
 import six
 from custom_inherit import DocInheritMeta
@@ -110,13 +108,15 @@ from numpy import ndarray
 
 from gemseo.core.dataset import Dataset
 from gemseo.mlearning.transform.transformer import Transformer
+from gemseo.utils.file_path_manager import FilePathManager
+from gemseo.utils.py23_compat import Path, xrange
 from gemseo.utils.string_tools import MultiLineString, pretty_repr
 
 LOGGER = logging.getLogger(__name__)
 
 TransformerType = Dict[str, Transformer]
 SavedObjectType = Union[Dataset, TransformerType, str, bool]
-DataType = Union[ndarray, Dict[str, ndarray]]
+DataType = Union[ndarray, Mapping[str, ndarray]]
 MLAlgoParameterType = Optional[Any]
 
 
@@ -124,6 +124,7 @@ MLAlgoParameterType = Optional[Any]
     DocInheritMeta(
         abstract_base_class=True,
         style="google_with_merge",
+        include_special_methods=True,
     )
 )
 class MLAlgo(object):
@@ -160,7 +161,7 @@ class MLAlgo(object):
 
     def __init__(
         self,
-        data,  # type:Dataset
+        data,  # type: Dataset
         transformer=None,  # type: Optional[TransformerType]
         **parameters  # type: MLAlgoParameterType
     ):  # type: (...) -> None
@@ -188,7 +189,9 @@ class MLAlgo(object):
             }
 
         self.algo = None
+        self.sizes = deepcopy(self.learning_set.sizes)
         self._trained = False
+        self._learning_samples_indices = xrange(len(self.learning_set))
 
     class DataFormatters(object):
         """Decorators for the internal MLAlgo methods."""
@@ -198,9 +201,14 @@ class MLAlgo(object):
         """Return whether the algorithm is trained."""
         return self._trained
 
+    @property
+    def learning_samples_indices(self):  # type: (...) -> Sequence[int]
+        """The indices of the learning samples used for the training."""
+        return self._learning_samples_indices
+
     def learn(
         self,
-        samples=None,  # type: List[int]
+        samples=None,  # type: Optional[Sequence[int]]
     ):  # type: (...) -> None
         """Train the machine learning algorithm from the learning dataset.
 
@@ -208,6 +216,20 @@ class MLAlgo(object):
             samples: The indices of the learning samples.
                 If None, use the whole learning dataset.
         """
+        if samples is not None:
+            self._learning_samples_indices = samples
+        self._learn(samples)
+        self._trained = True
+
+    def _learn(
+        self, indices  # type: Optional[Sequence[int]]
+    ):  # type: (...) -> None
+        """Define the indices of the learning samples.
+
+        Args:
+            indices: The indices of the learning samples.
+                If None, use the whole learning dataset.
+        """
         raise NotImplementedError
 
     def __str__(self):  # type: (...) -> str
@@ -216,13 +238,16 @@ class MLAlgo(object):
         msg.indent()
         if self.LIBRARY is not None:
             msg.add("based on the {} library", self.LIBRARY)
-        msg.add("built from {} learning samples", self.learning_set.length)
+        if self.is_trained:
+            msg.add(
+                "built from {} learning samples", len(self._learning_samples_indices)
+            )
         return str(msg)
 
     def save(
         self,
         directory=None,  # type: Optional[str]
-        path=".",  # type: str
+        path=".",  # type: Union[str,Path]
         save_learning_set=False,  # type: bool
     ):  # type: (...) -> str
         """Save the machine learning algorithm.
@@ -230,8 +255,8 @@ class MLAlgo(object):
         Args:
             directory: The name of the directory to save the algorithm.
             path: The path to parent directory where to create the directory.
-            save_learning_set: If False, do not save the learning set
-                to lighten the saved files.
+            save_learning_set: Whether to save the learning set
+                or get rid of it to lighten the saved files.
 
         Returns:
             The path to the directory where the algorithm is saved.
@@ -239,27 +264,25 @@ class MLAlgo(object):
         if not save_learning_set:
             self.learning_set.data = {}
             self.learning_set.length = 0
-        splitted_class_name = re.findall("[A-Z][a-z]*", self.__class__.__name__)
-        splitted_class_name = [word.lower() for word in splitted_class_name]
-        algo_name = "_".join(splitted_class_name)
-        algo_name += "_" + self.learning_set.name
-        directory = directory or algo_name
-        directory = join(path, directory)
-        if not exists(directory):
-            makedirs(directory)
 
-        filename = join(directory, self.FILENAME)
+        default_directory_name = "{}_{}".format(
+            FilePathManager.to_snake_case(self.__class__.__name__),
+            self.learning_set.name,
+        )
+        directory = Path(path) / (directory or default_directory_name)
+        directory.mkdir(exist_ok=True)
+
         objects = self._get_objects_to_save()
-        with open(filename, "wb") as handle:
+        with (directory / self.FILENAME).open("wb") as handle:
             pickle.dump(objects, handle)
 
         self._save_algo(directory)
 
-        return directory
+        return str(directory)
 
     def _save_algo(
         self,
-        directory,  # type: str
+        directory,  # type: Path
     ):  # type: (...) -> None
         """Save the interfaced machine learning algorithm.
 
@@ -267,13 +290,12 @@ class MLAlgo(object):
             directory: The path to the directory
                 where to save the interfaced machine learning algorithm.
         """
-        filename = join(directory, "algo.pkl")
-        with open(filename, "wb") as handle:
+        with (directory / "algo.pkl").open("wb") as handle:
             pickle.dump(self.algo, handle)
 
     def load_algo(
         self,
-        directory,  # type: str
+        directory,  # type: Union[str,Path]
     ):  # type: (...) -> None
         """Load a machine learning algorithm from a directory.
 
@@ -281,11 +303,8 @@ class MLAlgo(object):
             directory: The path to the directory
                 where the machine learning algorithm is saved.
         """
-
-        filename = join(directory, "algo.pkl")
-        with open(filename, "rb") as handle:
-            algo = pickle.load(handle)
-        self.algo = algo
+        with (Path(directory) / "algo.pkl").open("rb") as handle:
+            self.algo = pickle.load(handle)
 
     def _get_objects_to_save(self):  # type: (...) -> Dict[str,SavedObjectType]
         """Return the objects to save.
@@ -298,6 +317,7 @@ class MLAlgo(object):
             "transformer": self.transformer,
             "parameters": self.parameters,
             "algo_name": self.__class__.__name__,
+            "sizes": self.sizes,
             "_trained": self._trained,
         }
         return objects
